



if exist %SP_DIR%/pyarrow/tests/test_array.py exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %SP_DIR%/pyarrow/_cuda.cp%PY_VER:.=%-win_amd64.pyd exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
